var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

/*
 * This file is part of the nivo project.
 *
 * Copyright 2016-present, Raphaël Benitte.
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import React from 'react';
import { merge } from 'lodash-es';
import { TransitionMotion, spring } from 'react-motion';
import _ from 'lodash-es';
import compose from 'recompose/compose';
import withPropsOnChange from 'recompose/withPropsOnChange';
import withStateHandlers from 'recompose/withStateHandlers';
import pure from 'recompose/pure';
import { pack } from 'd3-hierarchy';
import { withHierarchy, withTheme, withColors, withDimensions, withMotion } from '../../../hocs';
import { extractRGB } from '../../../lib/colorUtils';
import noop from '../../../lib/noop';
import Container from '../Container';
import { getAccessorFor } from '../../../lib/propertiesConverters';
import { bubblePropTypes, bubbleDefaultProps } from './BubbleProps';

var ignoreProps = ['borderWidth', 'borderColor', 'enableLabel', 'label', 'labelFormat', 'labelTextColor', 'labelSkipRadius', 'labelTextDY', 'transitionDuration', 'transitionEasing'];

var computeNodeUID = function computeNodeUID(node, getIdentity) {
    return node.ancestors().map(function (ancestor) {
        return getIdentity(ancestor.data);
    }).join('.');
};

var nodeWillEnter = function nodeWillEnter(_ref) {
    var node = _ref.data;
    return _extends({
        r: 0,
        x: node.x,
        y: node.y
    }, extractRGB(node.color));
};

var nodeWillLeave = function nodeWillLeave(styleThatLeft) {
    return {
        r: spring(0),
        x: spring(styleThatLeft.data.x),
        y: spring(styleThatLeft.data.y)
    };
};

var computeZoom = function computeZoom(nodes, currentNodeUid, width, height) {
    var currentNode = nodes.find(function (_ref2) {
        var uid = _ref2.uid;
        return uid === currentNodeUid;
    });
    if (currentNode) {
        var ratio = Math.min(width, height) / (currentNode.r * 2);
        var offsetX = width / 2 - currentNode.x * ratio;
        var offsetY = height / 2 - currentNode.y * ratio;
        nodes.forEach(function (node) {
            node.r = node.r * ratio;
            node.x = node.x * ratio + offsetX;
            node.y = node.y * ratio + offsetY;
        });
    }
};

var BubblePlaceholders = function BubblePlaceholders(_ref3) {
    var root = _ref3.root,
        getIdentity = _ref3.getIdentity,
        leavesOnly = _ref3.leavesOnly,
        namespace = _ref3.namespace,
        pack = _ref3.pack,
        width = _ref3.width,
        height = _ref3.height,
        margin = _ref3.margin,
        outerWidth = _ref3.outerWidth,
        outerHeight = _ref3.outerHeight,
        theme = _ref3.theme,
        getColor = _ref3.getColor,
        animate = _ref3.animate,
        motionStiffness = _ref3.motionStiffness,
        motionDamping = _ref3.motionDamping,
        isInteractive = _ref3.isInteractive,
        children = _ref3.children,
        isZoomable = _ref3.isZoomable,
        zoomToNode = _ref3.zoomToNode,
        currentNodeUid = _ref3.currentNodeUid;

    // assign a unique id depending on node path to each node
    root.each(function (node) {
        node.id = getIdentity(node.data);
        node.uid = computeNodeUID(node, getIdentity);
    });

    pack(root);

    var nodes = leavesOnly ? root.leaves() : root.descendants();
    nodes = nodes.map(function (node) {
        node.color = getColor(_extends({}, node.data, { depth: node.depth }));
        // if (d.depth > 1) {
        //     d.color = color(d.parentId)
        // } else {
        //     d.color = color(identity(d.data))
        // }

        node.data.key = node.ancestors().map(function (a) {
            return getIdentity(a.data);
        }).join('.');

        return node;
    });

    if (currentNodeUid) computeZoom(nodes, currentNodeUid, width, height);

    var wrapperTag = void 0;
    var containerTag = void 0;

    var wrapperProps = {};
    var containerProps = {};

    if (namespace === 'svg') {
        wrapperTag = 'svg';
        containerTag = 'g';

        wrapperProps.width = outerWidth;
        wrapperProps.height = outerHeight;
        wrapperProps.xmlns = 'http://www.w3.org/2000/svg';
        containerProps.transform = 'translate(' + margin.left + ',' + margin.top + ')';
    } else {
        wrapperTag = 'div';
        containerTag = 'div';

        wrapperProps.style = {
            position: 'relative',
            width: outerWidth,
            height: outerHeight
        };
        containerProps.style = Object.assign({}, margin, {
            position: 'absolute'
        });
    }

    if (!animate) {
        return React.createElement(
            Container,
            { isInteractive: isInteractive, theme: theme },
            function (_ref4) {
                var showTooltip = _ref4.showTooltip,
                    hideTooltip = _ref4.hideTooltip;
                return React.createElement(wrapperTag, wrapperProps, React.createElement(containerTag, containerProps, children(nodes.map(function (node) {
                    return {
                        key: node.data.key,
                        data: node,
                        style: _.pick(node, ['r', 'x', 'y', 'color']),
                        zoom: isInteractive && isZoomable ? function () {
                            return zoomToNode(node.uid);
                        } : noop
                    };
                }), { showTooltip: showTooltip, hideTooltip: hideTooltip })));
            }
        );
    }

    var motionProps = {
        stiffness: motionStiffness,
        damping: motionDamping
    };

    return React.createElement(
        Container,
        { isInteractive: isInteractive, theme: theme },
        function (_ref5) {
            var showTooltip = _ref5.showTooltip,
                hideTooltip = _ref5.hideTooltip;
            return React.createElement(wrapperTag, wrapperProps, React.createElement(
                TransitionMotion,
                {
                    willEnter: nodeWillEnter,
                    willLeave: nodeWillLeave,
                    styles: nodes.map(function (b) {
                        return {
                            key: b.data.key,
                            data: b,
                            style: _extends({
                                r: spring(b.r, motionProps),
                                x: spring(b.x, motionProps),
                                y: spring(b.y, motionProps)
                            }, extractRGB(b.color, motionProps))
                        };
                    })
                },
                function (interpolatedStyles) {
                    return React.createElement(containerTag, containerProps, children(interpolatedStyles.map(function (interpolatedStyle) {
                        var _interpolatedStyle$st = interpolatedStyle.style,
                            colorR = _interpolatedStyle$st.colorR,
                            colorG = _interpolatedStyle$st.colorG,
                            colorB = _interpolatedStyle$st.colorB;

                        interpolatedStyle.style.color = 'rgb(' + Math.round(colorR) + ',' + Math.round(colorG) + ',' + Math.round(colorB) + ')';

                        if (isInteractive && isZoomable) {
                            interpolatedStyle.zoom = function () {
                                return zoomToNode(interpolatedStyle.data.uid);
                            };
                        } else {
                            interpolatedStyle.zoom = noop;
                        }

                        return interpolatedStyle;
                    }), { showTooltip: showTooltip, hideTooltip: hideTooltip }));
                }
            ));
        }
    );
};

BubblePlaceholders.propTypes = _.omit(bubblePropTypes, ignoreProps);

export var BubblePlaceholdersDefaultProps = _.omit(bubbleDefaultProps, ignoreProps);

BubblePlaceholders.defaultProps = BubblePlaceholdersDefaultProps;

var enhance = compose(withHierarchy(), withDimensions(), withTheme(), withMotion(), withColors({ defaultColorBy: 'depth' }), withPropsOnChange(['identity'], function (_ref6) {
    var identity = _ref6.identity;
    return {
        getIdentity: getAccessorFor(identity)
    };
}), withPropsOnChange(['width', 'height', 'padding'], function (_ref7) {
    var width = _ref7.width,
        height = _ref7.height,
        padding = _ref7.padding;
    return {
        pack: pack().size([width, height]).padding(padding)
    };
}), withStateHandlers(function (_ref8) {
    var _ref8$currentNodeUid = _ref8.currentNodeUid,
        currentNodeUid = _ref8$currentNodeUid === undefined ? null : _ref8$currentNodeUid;
    return {
        currentNodeUid: currentNodeUid
    };
}, {
    zoomToNode: function zoomToNode(_ref9) {
        var currentNodeUid = _ref9.currentNodeUid;
        return function (uid) {
            if (uid === currentNodeUid) {
                return { currentNodeUid: null };
            }

            return { currentNodeUid: uid };
        };
    }
}), pure);

export default enhance(BubblePlaceholders);